directory = {
    "b": "basic",
    "s": "standard",
    "m": "monorepo"
}

description = '''Creates a standard data science project directory. This helps in
                   easy team collaboration, rapid prototyping, easy reproducibility and fast iteration.       
                   The directory structure is by no means a globally recognized standard, but was inspired by
                   the folder structure created by the Azure team (https://docs.microsoft.com/en-us/azure/machine-learning/team-data-science-process/overview)
                 '''
version = '0.0.1'
